import tkinter as tk
from tkinter import filedialog, messagebox
from PIL import Image, ImageTk
import zipfile
import os
import requests
from io import BytesIO
from screeninfo import get_monitors
import shutil
import schedule
import time
import threading
import pytz
from datetime import datetime, timedelta


def load_url_from_file(viewer):
    try:
        with open("url.txt", "r") as file:
            url = file.read().strip()
            viewer.url_entry.insert(0, url)
    except FileNotFoundError:
        messagebox.showwarning("Warnung", "Die Datei 'url.txt' wurde nicht gefunden.")
    except Exception as e:
        messagebox.showerror("Fehler", f"Ein Fehler ist aufgetreten: {e}")


def save_url_to_file(viewer):
    try:
        url = viewer.url_entry.get()
        with open("url.txt", "w") as file:
            file.write(url)
    except Exception as e:
        messagebox.showerror("Fehler", f"Ein Fehler ist aufgetreten: {e}")


def copy_images(source_dir, dest_dir):
    if not os.path.exists(dest_dir):
        os.makedirs(dest_dir)
    for file_name in os.listdir(source_dir):
        if file_name.lower().endswith(('.png', '.jpg', '.jpeg', '.gif', '.bmp')):
            full_file_name = os.path.join(source_dir, file_name)
            if os.path.isfile(full_file_name):
                shutil.copy(full_file_name, dest_dir)


class ZipImageViewer:
    def __init__(self, root):
        self.root = root
        self.root.title("STI Simple Ads")

        # Bild anzeigen
        self.image = Image.open("STI_software.png")
        self.logo = ImageTk.PhotoImage(self.image)
        self.panel = tk.Label(root, image=self.logo)
        self.panel.grid(row=0, column=0, columnspan=2)

        # URL-Eingabefeld
        self.url_entry = tk.Entry(self.root, width=80)
        self.url_entry.grid(row=1, column=0, columnspan=2, pady=5)

        load_url_from_file(self)

        # Buttons
        self.download_button = tk.Button(self.root, text="Bilder von URL herunterladen",
                                         command=self.download_images_from_url, width=25)
        self.download_button.grid(row=2, column=0, padx=5)

        self.open_button = tk.Button(self.root, text="ZIP Datei öffnen", command=self.open_zip_file, width=25)
        self.open_button.grid(row=2, column=1, padx=5)

        self.CloseButton = tk.Button(text="Exit", command=self.quit_program, width=15)
        self.CloseButton.grid(row=8, column=1, pady=5, sticky=tk.E)

        # Bildschirmauswahl
        self.SelectScreen_Label = tk.Label(self.root, text="Bildschirm Wählen")
        self.SelectScreen_Label.grid(row=5, column=0, columnspan=2, pady=5)

        self.monitor_var = tk.StringVar(self.root)
        self.update_monitor_menu()  # Initialisiere das Dropdown-Menü für die Bildschirmauswahl

        # Anzeige-Button
        self.show_button = tk.Button(self.root, text="Anzeigen", command=self.show_images, state=tk.DISABLED,
                                     width=25)
        self.show_button.grid(row=7, column=0, columnspan=2, pady=5)

        # Bildwechsel-Intervall
        self.interval_scale = tk.Scale(self.root, from_=1, to=10, orient=tk.HORIZONTAL,
                                       label="Bildwechsel-Intervall (Sekunden)")
        self.interval_scale.grid(row=8, column=0, columnspan=2, pady=5)
        self.interval_scale.set(5)

        # Variableninitialisierung
        self.zip_file_path = ""
        self.extracted_images = []
        self.image_window = None

        # Tastenkombination zum Beenden des Programms
        self.root.bind("<Control-q>", self.quit_program)

        # Scheduling
        self.schedule_download()

    def quit_program(self, event=None):
        self.root.quit()

    def update_monitor_menu(self):
        # Entferne das alte Dropdown-Menü
        if hasattr(self, 'monitor_menu'):
            self.monitor_menu.destroy()

        # Fülle die Optionen neu basierend auf den aktuellen Monitoren
        monitors = get_monitors()
        monitor_names = [f"Monitor {i + 1}: {m.width}x{m.height}" for i, m in enumerate(monitors)]
        self.monitor_var.set(monitor_names[0])
        self.monitor_menu = tk.OptionMenu(self.root, self.monitor_var, *monitor_names)
        self.monitor_menu.grid(row=6, column=0, columnspan=2, pady=5)

    def download_images_from_url(self):
        url = self.url_entry.get()
        save_url_to_file(self)
        if not url:
            messagebox.showwarning("Fehler", "Bitte geben Sie eine URL ein.")
            return
        try:
            response = requests.get(url, timeout=300)  # Timeout von 300 Sekunden (5 Minuten)
            if response.status_code == 200:
                with open("downloaded_images.zip", "wb") as f:
                    f.write(response.content)
                self.zip_file_path = "downloaded_images.zip"
                self.extract_images()
                if self.extracted_images:
                    self.show_button.config(state=tk.NORMAL)
                    messagebox.showinfo("Erfolg", "Bilder erfolgreich heruntergeladen und entpackt!")
                    self.show_images()  # Automatically show images after successful download
                else:
                    messagebox.showwarning("Fehler", "TIMEOUT - Netzwerk Verbindung Prüfen.")
            else:
                messagebox.showerror("Fehler", f"Fehler beim Herunterladen der Datei: {response.status_code}")
        except requests.exceptions.Timeout:
            messagebox.showerror("Fehler", "Der Download hat zu lange gedauert und wurde abgebrochen.")
        except Exception as e:
            messagebox.showerror("Fehler", f"Ein Fehler ist aufgetreten: {e}")

    def open_zip_file(self):
        self.zip_file_path = filedialog.askopenfilename(filetypes=[("ZIP files", "*.zip")])
        if self.zip_file_path:
            self.extract_images()
            if self.extracted_images:
                self.show_button.config(state=tk.NORMAL)
                messagebox.showinfo("Erfolg", "Bilder erfolgreich entpackt!")
            else:
                messagebox.showwarning("Fehler", "Keine Bilder in der ZIP-Datei gefunden.")

    def extract_images(self):
        with zipfile.ZipFile(self.zip_file_path, 'r') as zip_ref:
            zip_ref.extractall("extracted_images")
        self.extracted_images = [os.path.join("extracted_images", f) for f in os.listdir("extracted_images") if
                                 f.lower().endswith(('.png', '.jpg', '.jpeg', '.gif', '.bmp'))]

        # Leeres "Current"-Verzeichnis erstellen oder existierendes leeren
        current_dir = "Current"
        if os.path.exists(current_dir):
            shutil.rmtree(current_dir)
        os.makedirs(current_dir)

        # Bilder aus "extracted_images" und "always" in "Current" kopieren
        copy_images("extracted_images", current_dir)
        copy_images("always", current_dir)

        # Wenn keine Bilder in "extracted_images" vorhanden sind, Bilder aus "Offline" in "Current" kopieren
        if not self.extracted_images:
            copy_images("Offline", current_dir)

        # Aktualisiere die Liste der extrahierten Bilder mit den Bildern im "Current"-Verzeichnis
        self.extracted_images = [os.path.join(current_dir, f) for f in os.listdir(current_dir) if
                                 f.lower().endswith(('.png', '.jpg', '.jpeg', '.gif', '.bmp'))]

    def show_images(self):
        if not self.extracted_images:
            messagebox.showwarning("Fehler", "Keine Bilder zum Anzeigen gefunden.")
            return

        # Ermitteln des ausgewählten Monitors
        selected_monitor_index = int(self.monitor_var.get().split()[1][:-1]) - 1
        selected_monitor = get_monitors()[selected_monitor_index]
        # screen_width = selected_monitor.width
        # screen_height = selected_monitor.height

        self.image_window = tk.Toplevel(self.root)
        self.image_window.title("Bilder anzeigen")

        # Ermittle die Abmessungen des ausgewählten Monitors
        selected_monitor_index = int(self.monitor_var.get().split()[1][:-1]) - 1
        selected_monitor = get_monitors()[selected_monitor_index]
        screen_width = selected_monitor.width
        screen_height = selected_monitor.height

        # Setze die Position und Größe des Toplevel-Fensters entsprechend den Abmessungen des ausgewählten Monitors
        self.image_window.geometry(f"{screen_width}x{screen_height}+{selected_monitor.x}+{selected_monitor.y}")
        self.image_window.attributes('-fullscreen', True)
        self.image_label = tk.Label(self.image_window)
        self.image_label.pack(expand=True)

        self.current_image_index = 0
        print(screen_width, screen_height)
        self.update_image(screen_width, screen_height)  # Übergeben Sie die Bildschirmabmessungen

    def update_image(self, screen_width, screen_height):
        if self.current_image_index < len(self.extracted_images):
            image_path = self.extracted_images[self.current_image_index]
            image = Image.open(image_path)
            image = image.resize((screen_width, screen_height), Image.LANCZOS)

            photo = ImageTk.PhotoImage(image)
            self.image_label.config(image=photo)
            self.image_label.image = photo
            self.current_image_index += 1
            self.root.after(self.interval_scale.get() * 1000,
                            lambda: self.update_image(screen_width, screen_height))  # Pass the arguments here
        else:
            self.current_image_index = 0
            self.update_image(screen_width, screen_height)

    def exit_fullscreen(self, event=None):
        self.image_window.attributes('-fullscreen', False)

    def schedule_download(self):
        # Schedule the download_images_from_url function to run at 01:00 German time every day
        def download_task():
            tz = pytz.timezone('Europe/Berlin')
            now = datetime.now(tz)
            if now.hour == 1 and now.minute == 0:
                self.download_images_from_url()

        schedule.every().day.at("01:00").do(download_task)

        # Create a separate thread to run the scheduler
        def run_scheduler():
            while True:
                schedule.run_pending()
                time.sleep(1)

        scheduler_thread = threading.Thread(target=run_scheduler)
        scheduler_thread.daemon = True  # Ensure the thread will exit when the main program exits
        scheduler_thread.start()


if __name__ == "__main__":
    root = tk.Tk()
    app = ZipImageViewer(root)
    root.mainloop()
